#include "widgets.h"
#include "objectdump.h"
#include <QDebug>

MainWindow::MainWindow()
:   m_timer(new QTimer(this))
,   m_flash(0)
{
#ifdef USE_QMAINWINDOW
    QWidget *centralWidget = new QWidget(this);
    m_flash = new FlashWidget(this, Qt::blue, Qt::yellow);
    QVBoxLayout *layout = new QVBoxLayout(this);
#   ifdef SET_MAINWINDOW_CONTENTS_MARGINS_TO_NONZERO
        setContentsMargins(10, 10, 10, 10);
#   endif
    layout->addWidget(m_flash);
    setCentralWidget(centralWidget);
    centralWidget->setLayout(layout);
#else
    QVBoxLayout *outerLayout = new QVBoxLayout(this);
#   ifdef SET_OUTERLAYOUT_CONTENTS_MARGINS_TO_ZERO
        outerLayout->setContentsMargins(0, 0, 0, 0);
#   endif
#   ifdef USE_INNER_WIDGET
        QWidget *innerWidget = new QWidget(this);
        QVBoxLayout *innerLayout = new QVBoxLayout(innerWidget);
        m_flash = new FlashWidget(innerWidget, Qt::blue, Qt::yellow);
        innerLayout->addWidget(m_flash);
        innerWidget->setLayout(innerLayout);
        outerLayout->addWidget(innerWidget);
#   else
        m_flash = new FlashWidget(this, Qt::blue, Qt::yellow);
        outerLayout->addWidget(m_flash);
#   endif
    setLayout(outerLayout);
#endif

    Q_ASSERT(connect(m_timer, SIGNAL(timeout()), this, SLOT(timerTick())));
    m_timer->start(1000);
}

void MainWindow::dump()
{
    QScopedPointer<ObjectDump::QVisitor> visitor(new ObjectDump::QVisitor);
    visitor->setPrefix("WinIdTest"); // to aid searchability of logs
    ObjectDump::addDefaultAnnotators(*visitor);
    ObjectDump::dumpTreeFromLeaf(*this, *visitor);
}

void MainWindow::timerTick()
{
    m_timer->stop();

    dump();

#ifdef CALL_WINID
    qDebug() << "WinIdTest: about to call winId() on m_flash";
    m_flash->winId();
#endif

    m_flash->setColor1(Qt::magenta);
    m_flash->setColor2(Qt::white);

    dump();
}


FlashWidget::FlashWidget(QWidget *parent, const QColor& color1, const QColor& color2)
:   QWidget(parent)
,   m_timer(new QTimer(this))
,   m_flag(false)
,   m_color1(color1)
,   m_color2(color2)
{
    Q_ASSERT(connect(m_timer, SIGNAL(timeout()), this, SLOT(timerTick())));
    m_timer->start(100);
}

void FlashWidget::timerTick()
{
    m_flag = !m_flag;
    update();
}

void FlashWidget::paintEvent(QPaintEvent *event)
{
    QPainter painter(this);
    const QColor &color = m_flag ? m_color1 : m_color2;
    painter.fillRect(event->rect(), color);
}




